@@ -8,7 +8,7 @@ from typing import TYPE_CHECKING
 
 from hypothesis import given
 from hypothesis.strategies import sampled_from
-from pytest import CaptureFixture, raises
+from pytest import CaptureFixture, mark, param, raises
 
 from utilities.iterables import one
 from utilities.traceback import (
@@ -17,10 +17,14 @@ from utilities.traceback import (
     format_exception_stack,
     make_except_hook,
 )
+from utilities.tzlocal import LOCAL_TIME_ZONE_NAME
+from utilities.whenever import get_now
 
 if TYPE_CHECKING:
     from collections.abc import Iterable
 
+    from utilities.types import MaybeCallableZonedDateTime
+
 
 class TestFormatExceptionStack:
     @classmethod
@@ -41,23 +45,27 @@ class TestFormatExceptionStack:
             result = format_exception_stack(error).splitlines()
             self._assert_lines(result)
 
-    def test_header(self) -> None:
+    @mark.parametrize("start", [param(get_now), param(None)])
+    def test_header(self, *, start: MaybeCallableZonedDateTime) -> None:
         try:
             _ = self.func(1, 2, 3, 4, c=5, d=6, e=7)
         except AssertionError as error:
-            result = format_exception_stack(error, header=True).splitlines()
+            result = format_exception_stack(
+                error, header=True, start=start
+            ).splitlines()
             patterns = [
-                r"^Date/time \| .+$",
-                r"^Started   \| .+$",
-                r"^Duration  \| .+$",
-                r"^User      \| .+$",
-                r"^Host      \| .+$",
-                r"^Version   \|\s$",
+                rf"^Date/time  \| \d{{8}}T\d{{6}}\[{LOCAL_TIME_ZONE_NAME}\]$",
+                rf"^Started    \| (\d{{8}}T\d{{6}}\[{LOCAL_TIME_ZONE_NAME}\]|)$",
+                r"^Duration   \| (-?PT\d+\.\d+S|)$",
+                r"^User       \| .+$",
+                r"^Host       \| .+$",
+                r"^Process ID \| \d+$",
+                r"^Version    \|\s$",
                 r"^$",
             ]
-            for line, pattern in zip(result[:7], patterns[:7], strict=False):
+            for line, pattern in zip(result[:8], patterns, strict=False):
                 assert search(pattern, line), line
-            self._assert_lines(result[7:])
+            self._assert_lines(result[len(patterns) :])
 
     def test_capture_locals(self) -> None:
         try:
