@@ -13,7 +13,7 @@ from utilities.iterables import always_iterable
 from utilities.logging import get_logger
 from utilities.os import temp_environ
 from utilities.pathlib import ensure_suffix
-from utilities.sqlalchemy import get_table_name
+from utilities.sqlalchemy import extract_url, get_table_name
 from utilities.timer import Timer
 from utilities.types import PathLike
 
@@ -37,6 +37,8 @@ async def pg_dump(
     data_only: bool = False,
     clean: bool = False,
     create: bool = False,
+    extension: MaybeListStr | None = None,
+    extension_exc: MaybeListStr | None = None,
     schema: MaybeListStr | None = None,
     schema_exc: MaybeListStr | None = None,
     table: MaybeSequence[TableOrORMInstOrClass | str] | None = None,
@@ -59,6 +61,8 @@ async def pg_dump(
         data_only=data_only,
         clean=clean,
         create=create,
+        extension=extension,
+        extension_exc=extension_exc,
         schema=schema,
         schema_exc=schema_exc,
         table=table,
@@ -70,7 +74,7 @@ async def pg_dump(
     )
     if dry_run:
         if logger is not None:
-            get_logger(logger=logger).info("Would run %r", str(cmd))
+            get_logger(logger=logger).info("Would run:\n\t%r", str(cmd))
         return
     with temp_environ(PGPASSWORD=url.password), Timer() as timer:  # pragma: no cover
         try:
@@ -109,6 +113,8 @@ def _build_pg_dump(
     data_only: bool = False,
     clean: bool = False,
     create: bool = False,
+    extension: MaybeListStr | None = None,
+    extension_exc: MaybeListStr | None = None,
     schema: MaybeListStr | None = None,
     schema_exc: MaybeListStr | None = None,
     table: MaybeSequence[TableOrORMInstOrClass | str] | None = None,
@@ -118,12 +124,11 @@ def _build_pg_dump(
     role: str | None = None,
     docker: str | None = None,
 ) -> str:
-    database, host, port = _extract_url(url)
+    extracted = extract_url(url)
     path = _path_pg_dump(path, format_=format_)
     parts: list[str] = [
         "pg_dump",
         # general options
-        f"--dbname={database}",
         f"--file={str(path)!r}",
         f"--format={format_}",
         "--verbose",
@@ -133,14 +138,22 @@ def _build_pg_dump(
         "--no-owner",
         "--no-privileges",
         # connection options
-        f"--host={host}",
-        f"--port={port}",
+        f"--dbname={extracted.database}",
+        f"--host={extracted.host}",
+        f"--port={extracted.port}",
+        f"--username={extracted.username}",
         "--no-password",
     ]
     if (format_ == "directory") and (jobs is not None):
         parts.append(f"--jobs={jobs}")
     if create:
         parts.append("--create")
+    if extension is not None:
+        parts.extend([f"--extension={e}" for e in always_iterable(extension)])
+    if extension_exc is not None:
+        parts.extend([
+            f"--exclude-extension={e}" for e in always_iterable(extension_exc)
+        ])
     if schema is not None:
         parts.extend([f"--schema={s}" for s in always_iterable(schema)])
     if schema_exc is not None:
@@ -155,8 +168,6 @@ def _build_pg_dump(
         parts.append("--inserts")
     if on_conflict_do_nothing:
         parts.append("--on-conflict-do-nothing")
-    if url.username is not None:
-        parts.append(f"--username={url.username}")
     if role is not None:
         parts.append(f"--role={role}")
     if docker is not None:
@@ -191,9 +202,9 @@ async def restore(
     /,
     *,
     psql: bool = False,
-    database: str | None = None,
     data_only: bool = False,
     clean: bool = False,
+    create: bool = False,
     jobs: int | None = None,
     schema: MaybeListStr | None = None,
     schema_exc: MaybeListStr | None = None,
@@ -208,9 +219,9 @@ async def restore(
         url,
         path,
         psql=psql,
-        database=database,
         data_only=data_only,
         clean=clean,
+        create=create,
         jobs=jobs,
         schema=schema,
         schema_exc=schema_exc,
@@ -220,7 +231,7 @@ async def restore(
     )
     if dry_run:
         if logger is not None:
-            get_logger(logger=logger).info("Would run %r", str(cmd))
+            get_logger(logger=logger).info("Would run:\n\t%r", str(cmd))
         return
     with temp_environ(PGPASSWORD=url.password), Timer() as timer:  # pragma: no cover
         try:
@@ -256,7 +267,6 @@ def _build_pg_restore_or_psql(
     /,
     *,
     psql: bool = False,
-    database: str | None = None,
     data_only: bool = False,
     clean: bool = False,
     create: bool = False,
@@ -269,11 +279,10 @@ def _build_pg_restore_or_psql(
 ) -> str:
     path = Path(path)
     if (path.suffix == ".sql") or psql:
-        return _build_psql(url, path, database=database, docker=docker)
+        return _build_psql(url, path, docker=docker)
     return _build_pg_restore(
         url,
         path,
-        database=database,
         data_only=data_only,
         clean=clean,
         create=create,
@@ -291,7 +300,6 @@ def _build_pg_restore(
     path: PathLike,
     /,
     *,
-    database: str | None = None,
     data_only: bool = False,
     clean: bool = False,
     create: bool = False,
@@ -303,12 +311,10 @@ def _build_pg_restore(
     docker: str | None = None,
 ) -> str:
     """Run `pg_restore`."""
-    url_database, host, port = _extract_url(url)
-    database_use = url_database if database is None else database
+    extracted = extract_url(url)
     parts: list[str] = [
         "pg_restore",
         # general options
-        f"--dbname={database_use}",
         "--verbose",
         # restore options
         *_resolve_data_only_and_clean(data_only=data_only, clean=clean),
@@ -316,8 +322,10 @@ def _build_pg_restore(
         "--no-owner",
         "--no-privileges",
         # connection options
-        f"--host={host}",
-        f"--port={port}",
+        f"--host={extracted.host}",
+        f"--port={extracted.port}",
+        f"--username={extracted.username}",
+        f"--dbname={extracted.database}",
         "--no-password",
     ]
     if create:
@@ -330,8 +338,6 @@ def _build_pg_restore(
         parts.extend([f"--exclude-schema={s}" for s in always_iterable(schemas_exc)])
     if tables is not None:
         parts.extend([f"--table={_get_table_name(t)}" for t in always_iterable(tables)])
-    if url.username is not None:
-        parts.append(f"--username={url.username}")
     if role is not None:
         parts.append(f"--role={role}")
     if docker is not None:
@@ -340,29 +346,20 @@ def _build_pg_restore(
     return " ".join(parts)
 
 
-def _build_psql(
-    url: URL,
-    path: PathLike,
-    /,
-    *,
-    database: str | None = None,
-    docker: str | None = None,
-) -> str:
+def _build_psql(url: URL, path: PathLike, /, *, docker: str | None = None) -> str:
     """Run `psql`."""
-    url_database, host, port = _extract_url(url)
-    database_use = url_database if database is None else database
+    extracted = extract_url(url)
     parts: list[str] = [
         "psql",
         # general options
-        f"--dbname={database_use}",
+        f"--dbname={extracted.database}",
         f"--file={str(path)!r}",
         # connection options
-        f"--host={host}",
-        f"--port={port}",
+        f"--host={extracted.host}",
+        f"--port={extracted.port}",
+        f"--username={extracted.username}",
         "--no-password",
     ]
-    if url.username is not None:
-        parts.append(f"--username={url.username}")
     if docker is not None:
         parts = _wrap_docker(parts, docker)
     return " ".join(parts)
@@ -371,42 +368,6 @@ def _build_psql(
 ##
 
 
-def _extract_url(url: URL, /) -> tuple[str, str, int]:
-    if url.database is None:
-        raise _ExtractURLDatabaseError(url=url)
-    if url.host is None:
-        raise _ExtractURLHostError(url=url)
-    if url.port is None:
-        raise _ExtractURLPortError(url=url)
-    return url.database, url.host, url.port
-
-
-@dataclass(kw_only=True, slots=True)
-class ExtractURLError(Exception):
-    url: URL
-
-
-@dataclass(kw_only=True, slots=True)
-class _ExtractURLDatabaseError(ExtractURLError):
-    @override
-    def __str__(self) -> str:
-        return f"Expected URL to contain a 'database'; got {self.url}"
-
-
-@dataclass(kw_only=True, slots=True)
-class _ExtractURLHostError(ExtractURLError):
-    @override
-    def __str__(self) -> str:
-        return f"Expected URL to contain a 'host'; got {self.url}"
-
-
-@dataclass(kw_only=True, slots=True)
-class _ExtractURLPortError(ExtractURLError):
-    @override
-    def __str__(self) -> str:
-        return f"Expected URL to contain a 'port'; got {self.url}"
-
-
 def _get_table_name(obj: TableOrORMInstOrClass | str, /) -> str:
     match obj:
         case Table() | DeclarativeBase() | type() as table_or_orm:
@@ -444,4 +405,4 @@ def _wrap_docker(parts: list[str], container: str, /) -> list[str]:
     return ["docker", "exec", "-it", container, *parts]
 
 
-__all__ = ["ExtractURLError", "pg_dump", "restore"]
+__all__ = ["pg_dump", "restore"]
